/*
 Copyright 2004, Revolution Linux Inc., Nicolas Dufresne, Yves Trudeau

 This file is part of the MILLE-XTERM distribution.
 See the MILLE-XTERM (english) and/or the MILLE (french) project web site

 http://www.revolutionlinux.com/mille-xterm/
 http://www.mille.ca/

 The MILLE-XTERM framework is covered by the GNU General Public License. See
 the COPYING file in the top-level MILLE-XTERM directory. Software packages
 that are included in the MILLE-XTERM distribution have their own licenses.

 -------------------------------------------------------------------------
 
 This script creates the tables needed for the Mille-XTERM configurator.

 Revisions history:
	- 13 jul 2004: Creation

*/

/* The nodes table contains the tree of nodes, an entry
   can be a node or a leaf depending of the value of isNode. If the 
   entry is a node, the field MAC is not use and if the entry is a leaf,
   the field name is not used.

   id_previous is a circular reference to id. The root of the tree has id = 0.

 - id_previous is the ancestor in the tree (root = 0)
 - groupname is the name of the group when isGroup = true
 - MAC is the mac address of the node when isGroup = false
*/
CREATE TABLE nodes (
	id SERIAL PRIMARY KEY,
	id_previous INTEGER NOT NULL REFERENCES nodes (id),
	name VARCHAR(50) DEFAULT '',
	mac VARCHAR(17) DEFAULT '00:00:00:00:00:00',
	isNode INTEGER DEFAULT 1
	CONSTRAINT invalidName CHECK (isNode = 0 OR (LENGTH(name) > 0))
	CONSTRAINT invalidMac  CHECK (isNode = 1 OR (mac > '00:00:00:00:00:00'))
	CONSTRAINT invalidIsNode CHECK (isNode = 1 OR isNode = 0)
);
CREATE INDEX nodes_name_IDX ON nodes (name); 
CREATE INDEX nodes_mac_IDX ON nodes (mac); 
CREATE INDEX nodes_isNode_IDX ON nodes (isNode); 

-- nodes needs a root node_group with id = 0
INSERT INTO nodes (id,id_previous,name) VALUES (0,0,'RootNode');
INSERT INTO nodes (id,id_previous,name) VALUES (-1,0,'Trash');

/* The attributesDef table contains the definition of the attributes

 - name is the name of the attribute
 - attributeType is the type of attribute: 0 = String (default), 1 = mask, 2 = list.
   When attributeType = 2 (list), the possible values are found in attributesDefDict
 - mask is the mask to apply to the value in regular expression
*/
CREATE TABLE attributesDef (
	id SERIAL PRIMARY KEY,
	name VARCHAR(50) UNIQUE,
	attributeType INTEGER DEFAULT 0 check( attributeType >=0 AND attributeType <= 2) ,
	mask VARCHAR(254) DEFAULT '^[^\']{0,254}$');
CREATE INDEX attributesDef_name_IDX ON attributesDef (name); 

/* The attributes table contains the attributes of a group or a node

 - nodes_id is the nodes owner of this attributes
 - attributesDef_id is the definition of the attributes
 - value is the actual value of the attributes
*/
CREATE TABLE attributes (
	id SERIAL UNIQUE,
	nodes_id INTEGER NOT NULL REFERENCES nodes (id),
	attributesDef_id INTEGER NOT NULL REFERENCES attributesDef (id),
	value VARCHAR(254),
	PRIMARY KEY (nodes_id, attributesDef_id));

/* The attributesDefDict contains the possible values of an attributes when
   its attributeType = 1 (list)

 - attributesDef_id is the attribute definition of type list owner of the value
 - sortval is a hint for sorting the list
*/
CREATE TABLE attributesDefDict (
	attributesDef_id INTEGER NOT NULL REFERENCES attributesDef (id),
	value VARCHAR(254),
	sortval INTEGER,
	PRIMARY KEY (attributesDef_id,value));
CREATE INDEX attributesDefDict_attributes_id_IDX ON attributesDefDict (attributesDef_id); 

/* The attributesSelect contains results from a type 2 attributesType (muliselect list)
	
	- attributes_id references an attributes
	- attributesDef_id references attributes definition
	- attributesDefDict_id references an element from attributes definition dictionary
	
*/
CREATE TABLE attributesSelect (
	attributes_id INTEGER REFERENCES attributes (id),
	attributesDef_id INTEGER, 
	value VARCHAR(254), 
	FOREIGN KEY (attributesDef_id,value) REFERENCES attributesDefDict (attributesDef_id,value),
	PRIMARY KEY (attributes_id,attributesDef_id,value));

/* The status table store actual status of leaves.

 - mac, the mac address of the node accessing the configurator (mandatory)
 - nodeIpAdd, the ip address of the node accessing the configurator if available
 - nfsIp, the ip address of the nfs server
 - xServIp, the ip address of the X server (needed for code = 2 and 3) 
 - display, the X display number of the leave accessing the configurator
   if available
 - username, the name of the user on the node accessing the configurator
   if available
 - code, the request asked to the configurator. Currently supported values 
   are:
	1: Node asks for config (giving ip and nfsIp)
	2: Starting X session for display manager
	3: Starting X session for user session
 - lastNfsIp, timestamp of last nfsIp modification
 - lastXServIp, timestamp of last nfsIp modification
*/
CREATE TABLE status (
	id INTEGER NOT NULL PRIMARY KEY REFERENCES nodes (id),
	mac VARCHAR(17) NOT NULL CONSTRAINT macNotUnique UNIQUE,
	ip VARCHAR(15) NOT NULL CONSTRAINT ipNotUnique UNIQUE,
	bootServIp VARCHAR(15) NOT NULL,
	appServIp VARCHAR(15) DEFAULT '0.0.0.0',
	display VARCHAR(50) DEFAULT '-1',
	username VARCHAR(50) DEFAULT '',
	code INTEGER NOT NULL CHECK (code > 0 AND code <= 4),
	bootServRetry INTEGER DEFAULT 0,
	appServRetry INTEGER DEFAULT 0,
	ts_lastBootServIp TIMESTAMP DEFAULT 'now',
	ts_lastAppServIp TIMESTAMP DEFAULT '-infinity'
	CONSTRAINT invalidAppServIp CHECK (code = 1 OR (code > 1 AND appServIP > '0.0.0.0'))
	CONSTRAINT invalidDisplay CHECK (code = 1 OR code= 2 OR (code > 2 AND LENGTH(display) > 0 AND display != '-1'))
	CONSTRAINT invalidUsername CHECK (code < 3 OR (code >= 3 AND LENGTH(username) > 0))
	CONSTRAINT notALeave CHECK (mac > '00:00:00:00:00:00'));
CREATE INDEX status_id_IDX ON status (id); 
CREATE INDEX status_ip_IDX ON status (ip);
CREATE INDEX status_appServIp_IDX ON status (appServIp); 
CREATE INDEX status_bootServIp_IDX ON status (bootServIp); 
CREATE INDEX status_username_IDX ON status (username); 
CREATE INDEX status_code_IDX ON status (code);  

	
	
/* The conLog table store the log of access to the configurator.

 - mac, the mac address of the node accessing the configurator (mandatory)
 - ip, the ip address of the node accessing the configurator if available
 - nfsIp, the ip address of the nfs server
 - xServIp, the ip address of the X server (needed for code = 2 and 3) 
 - display, the X display number of the leave accessing the configurator
   if available
 - username, the name of the user on the node accessing the configurator
   if available
 - code, the request asked to the configurator. Currently supported values 
   are:
	1: Node asks for config (giving ip, nfsIp
	2: Starting X session for display manager
	3: Starting X session for user session


*/
CREATE TABLE log (
	id_key SERIAL PRIMARY KEY,
	id INTEGER NOT NULL,
	mac VARCHAR(17) NOT NULL,
	ip VARCHAR(15) NOT NULL,
	bootServIp VARCHAR(15) NOT NULL,
	appServIp VARCHAR(15) DEFAULT '0.0.0.0',
	display INTEGER DEFAULT -1,
	username VARCHAR(50) DEFAULT '',
	code INTEGER NOT NULL CHECK (code > 0 AND code <= 4),
	bootServRetry INTEGER DEFAULT 0,
	appServRetry INTEGER DEFAULT 0,
	ts TIMESTAMP DEFAULT 'now'
	CONSTRAINT invalidAppServIp CHECK (code = 1 OR (code > 1 AND appServIP > '0.0.0.0'))
	CONSTRAINT invalidDisplay CHECK (code = 1 OR code= 2 OR (code > 2 AND LENGTH(display) > 0 AND display != '-1'))
	CONSTRAINT invalidUsername CHECK (code < 3 OR (code >= 3 AND LENGTH(username) > 0))
	CONSTRAINT notALeave CHECK (mac > '00:00:00:00:00:00'));
CREATE INDEX log_mac_IDX ON log (mac); 
CREATE INDEX log_ip_IDX ON log (ip); 
CREATE INDEX log_appServIp_IDX ON log (appServIp); 
CREATE INDEX log_bootServIp_IDX ON log (bootServIp); 
CREATE INDEX log_username_IDX ON log (username); 
CREATE INDEX log_TS_IDX ON log (ts);  
